/*
 * fancyqt4asyncbroadcasthandler.h
 *
 *  Created on: 11.05.2010
 *      Author: lech
 */

#ifndef FANCYQT4ASYNCBROADCASTHANDLER_H_
#define FANCYQT4ASYNCBROADCASTHANDLER_H_

#include <fancyqt4.h>
#include <fancyobject.h>

#include <fancysock2/fancysockonewayhandler.h>

class QObject;

namespace Fancy {

class Map;
namespace Sock {
class InvocationContext;
}
namespace Qt4 {

class OneWayInvocationCallEmitter;

/**
 * This object is used to dispatch a call to @c Fancy::Sock::OneWayHandler.doOneWay()
 * via Qt4's asynchronous signal delivery mechanism.
 *
 * It becomes active through registering it with the client pool, like this:
 * AsyncBroadcastHandler::ptr_t asyncBroadcastHandler = AsyncBroadcastHandler::newInstance();
 * asyncBroadcastHandler->connectOneWayInvocationCallReceived(foo, SLOT(oneWayInvocationCallReceived(Fancy::Qt4::OneWayInvocationCall::ptr_t&)));
 * remotingClientPool->registerBroadcastHandler(asyncBroadcastHandler);
 */
class FANCYQT4_API AsyncBroadcastHandler : public virtual Fancy::Sock::OneWayHandler {

public:

    typedef Fancy::ObjPtr<AsyncBroadcastHandler> ptr_t;

    /**
     * Construct a AsyncBroadcastHandler
     */
    static ptr_t newInstance();

    /**
     * A shortcut for
     * <pre>
     *  QObject::connect(this->getEmitter(),
     *                   SIGNAL(oneWayInvocationCallReceived(const Fancy::Qt4::OneWayInvocationCall::ptr_t&)),
     *                   receiver,
     *                   method);
     * </pre>
     *
     * Please note, that due to limitations of Qt's asynchronous signal/slot handling, you should
     * always include @c fancyqt4onewayinvocationcall.h in your header files and use the full
     * @c Fancy::Qt4::OneWayInvocationCall::ptr_t notation instead of i.e.
     * @c Fancy:.ObjPtr<Fancy::Qt4::OneWayInvocationCall>
     *
     * @param receiver The receiving Object, in most cases a GUI widget.
     * @param method The receiving slot which should be constructed by the @c SLOT macro.
     * @return the return value of QObject::connect()
     */
    virtual bool connectOneWayInvocationCallReceived(QObject *receiver, const char * method) const = 0;

    /**
     * @return The QObject, which's signals are emitted.
     */
    virtual OneWayInvocationCallEmitter *getEmitter() const = 0;

    /** this fullfills the interface Fancy::Sock::OneWayHandler */
    virtual void doOneWay (const Fancy::ObjPtr<Fancy::Sock::InvocationContext> &ctx,
                           const Object::ptr_t &arg,
                           const Fancy::ObjPtr<Fancy::Map> &metadata) = 0;
};

}} // end namespaces

#endif /* FANCYQT4ASYNCBROADCASTHANDLER_H_ */
