/*
 * fancyqt4invocationresult.h
 *
 *  Created on: 10.12.2009
 *      Author: wglas
 */

#ifndef FANCYQT4INVOCATIONRESULT_H_
#define FANCYQT4INVOCATIONRESULT_H_

#include <fancyqt4.h>
#include <fancyobject.h>
#include <fancyexception.h>
#include <fancythrowable.h>

#include <string>

namespace Fancy {

class Throwable;

namespace Qt4 {

/**
 * Common base class for @c InvocationResult and @c InvocationException.
 */
class InvocationResultSupport : public Fancy::Object
{
    std::string bean;
    std::string method;
    Fancy::Object::ptr_t context;
public:
    typedef Fancy::ObjPtr<InvocationResultSupport> ptr_t;

    /**
     * @param bean The bean called.
     * @param method The name of the method called.
     * @param context The client-defined context set via @c AsyncInvocationHandler::setContext().
     * @return A new @c InvocationResultSupport instance.
     */
    static ptr_t newInstance(const std::string &bean,
                             const std::string &method,
                             const Fancy::Object::ptr_t &context);

    /**
     * @return The bean name called.
     */
    std::string getBean() const {
        return this->bean;
    }

    /**
     * @return The client-defined context set via @c AsyncInvocationHandler::setContext().
     */
    Fancy::Object::ptr_t getContext() const {
        return this->context;
    }

    /**
     * @return The name of the method called.
     */
    std::string getMethod() const {
        return this->method;
    }

protected:
    InvocationResultSupport(const std::string &_bean,
                            const std::string &_method,
                            const Fancy::Object::ptr_t &_context);
    virtual ~InvocationResultSupport();
};

/**
 * An invocation result with a return value.
 */
class FANCYQT4_API InvocationResult: public InvocationResultSupport {

    Fancy::Object::ptr_t result;

public:
    typedef Fancy::ObjPtr<InvocationResult> ptr_t;

    /**
     * @param result The result of the invocation.
     * @param bean The bean called.
     * @param method The name of the method called.
     * @param context The client-defined context set via @c AsyncInvocationHandler::setContext().
     * @return A new @c InvocationResultSupport instance.
     */
    static ptr_t newInstance(const Fancy::Object::ptr_t &result,
                             const std::string &bean,
                             const std::string &method,
                             const Fancy::Object::ptr_t &context);

    /**
     * @return The result returned by the invocation.
     */
    Fancy::Object::ptr_t getResult() const {
        return this->result;
    }

protected:
    InvocationResult(const Fancy::Object::ptr_t &_result,
                     const std::string &_bean,
                     const std::string &_method,
                     const Fancy::Object::ptr_t &_context);
    virtual ~InvocationResult();
};

/**
 * An exception as thrown by an invocation.
 */
class FANCYQT4_API InvocationException: public InvocationResultSupport {

    Fancy::Exception exception;
    Fancy::ObjPtr<Fancy::Throwable> throwable;

public:
    typedef Fancy::ObjPtr<InvocationException> ptr_t;

    /**
     * @param exception The exception thrown by the invocation.
     * @param bean The bean called.
     * @param method The name of the method called.
     * @param context The client-defined context set via @c AsyncInvocationHandler::setContext().
     * @return A new @c InvocationResultSupport instance.
     */
    static ptr_t newInstance(const Fancy::Exception &exception,
                             const std::string &bean,
                             const std::string &method,
                             const Fancy::Object::ptr_t &context);

    /**
     * @return The exception thrown by the invocation.
     */
    const Fancy::Exception &getException() const {
        return this->exception;
    }

    Fancy::ObjPtr<Fancy::Throwable> getThrowable() const {
	  return this->throwable;
	}

protected:
    InvocationException(const Fancy::Exception &_exception,
                        const std::string &_bean,
                        const std::string &_method,
                        const Fancy::Object::ptr_t &_context);

    InvocationException(const Fancy::Exception &_exception,
                        const std::string &_bean,
                        const std::string &_method,
                        const Fancy::Object::ptr_t &_context,
						const Fancy::ObjPtr<Fancy::Throwable> &_throwable);

    virtual ~InvocationException();
};

}

}

#endif /* FANCYQT4INVOCATIONRESULT_H_ */
