/*
 * fancyqt4invocationresultemitter.h
 *
 *  Created on: 08.12.2009
 *      Author: wglas
 */

#ifndef FANCYQT4INVOCATIONRESULTEMITTER_H_
#define FANCYQT4INVOCATIONRESULTEMITTER_H_

#include <fancyqt4invocationresult.h>
#include <QtCore/qobject.h>
#include <fancyobject.h>

namespace Fancy {

class Exception;

namespace Qt4 {

/**
 * An object, which allows to propagate the result of an asynchronous invocation to a GUI component.
 *
 * The main feature of this component is, that the signals of this object may be emitted in a
 * worker thread and delivered to a slot of a GUI component, which uis executed inside the GUI thread.
 *
 * Moreover, the design of Qt's signal/slot mechanism assures, that the slot is not called, if the
 * GUI component is destroyed before the synchronous invocation finishes.
 */
class FANCYQT4_API InvocationResultEmitter: public QObject {

    Q_OBJECT

public:
    /**
     * Construct a result emitter.
     * @param parent An optional parent object. This pointer will most likely be
     *               set to @c null and the lifecycle will be controlled by
     *               a FAncy::Object instance, which lifes as long as pending invocations
     *               are around.
     * @return A new instance.
     */
    InvocationResultEmitter(QObject * parent = 0);

    virtual ~InvocationResultEmitter();

    /**
     * This signal is emitted, when an asynchronous operation finished.
     *
     * @param result The result of the invocation.
     */
    Q_SIGNAL void resultReceived(const Fancy::Qt4::InvocationResult::ptr_t &result);

    /**
     * This signal is emitted, when an asynchronous operation threw an exception.
     *
     * @param method The method called.
     * @param exception The exception thrown by the invocation.
     */
    Q_SIGNAL void exceptionReceived(const Fancy::Qt4::InvocationException::ptr_t &exception);
};

}

}

#endif /* FANCYQT4INVOCATIONRESULTEMITTER_H_ */
