/*
 * String.h
 *
 *  Created on: 03.12.2009
 *      Author: wglas
 */

#ifndef FANCYQT4STRING_H_
#define FANCYQT4STRING_H_

#include <fancyqt4.h>
#include <QtCore/qstring.h>
#include <fancystring.h>

namespace Fancy {

namespace Qt4 {

class StringHelper;

class FANCYQT4_API String : public virtual Fancy::String {

    QString s;
    mutable Fancy::String::ptr_t utf8;
    friend class StringHelper;

public:

	static Fancy::String::ptr_t newInstance();
	static Fancy::String::ptr_t newInstance(const QString &s);
	static Fancy::String::ptr_t newInstance(const std::string &s);
	static Fancy::String::ptr_t newInstance(const char *s);
	static Fancy::Serializable::ptr_t newSerializableInstance();

	/**
	 * Register @c Fancy::Qt4::String as the default implementation of
	 * @c Fancy::String. This method uses @c Fancy::String::registerImplementation
	 * and should be called at the very beginning of you @c main function.
	 */
	static void registerAsDefaultString();

    static QString toQString(const Fancy::Object::ptr_t &o);

    /**
     * Format a @c std::string using a printf-style format string and stdarg
     * argument list like in @c vsnprintf.
     */
#ifdef __GNUG__
      __attribute__((format(printf,1,2)))
#endif
    static QString sprintf(const char *fmt, ...);

    /**
     * Format a @c std::string using a printf-style format string and a
     * arguments.
     */
    static QString vsprintf(const char *fmt, va_list ap);

	QString getQString() const;

	std::string getUtf8String() const;

	size_t getUtf8Length() const;

    std::string toString() const;

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    void writeFields(const Fancy::ObjPtr<Fancy::Serializer> &serializer) const;
    ObjPtr<Serializable> readFields(const Fancy::ObjPtr<Fancy::Serializer> &serializer, int64_t serialVersionUID);

protected:
	String();
	String(const QString &s);
    String(const QString &s, const std::string &utf8);
	~String();
};

}
}

#endif /* FANCYQT4STRING_H_ */
